<?php
/*----------------------------------------------------------------------------------|  simontech.me  |----/
				Award Manager by SimonTech Development
/-------------------------------------------------------------------------------------------------------/

	@version		1.0.7
	@build			8th June, 2019
	@created		19th April, 2019
	@package		Award Manager
	@subpackage		nominees.php
	@author			SimonTech <https://simontech.me>
	@copyright		Copyright (C) 2019 SimonTech Development. All Rights Reserved
	@license		GNU/GPL Version 2 or later - http://www.gnu.org/licenses/gpl-2.0.html
  ____  _____  _____  __  __  __      __       ___  _____  __  __  ____  _____  _  _  ____  _  _  ____
 (_  _)(  _  )(  _  )(  \/  )(  )    /__\     / __)(  _  )(  \/  )(  _ \(  _  )( \( )( ___)( \( )(_  _)
.-_)(   )(_)(  )(_)(  )    (  )(__  /(__)\   ( (__  )(_)(  )    (  )___/ )(_)(  )  (  )__)  )  (   )(
\____) (_____)(_____)(_/\/\_)(____)(__)(__)   \___)(_____)(_/\/\_)(__)  (_____)(_)\_)(____)(_)\_) (__)

/------------------------------------------------------------------------------------------------------*/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Award_manager Model for Nominees
 */
class Award_managerModelNominees extends JModelList
{
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
        {
			$config['filter_fields'] = array(
				'a.catid','category'
			);
		}

		parent::__construct($config);
	}

	/**
	 * Model user data.
	 *
	 * @var        strings
	 */
	protected $user;
	protected $userId;
	protected $guest;
	protected $groups;
	protected $levels;
	protected $app;
	protected $input;
	protected $uikitComp;


	public function getTable($type = 'nominee', $prefix = 'Award_managerTable', $config = array())
  {
    // add table path for when model gets used from other component
    $this->addTablePath(JPATH_ADMINISTRATOR . '/components/com_award_manager/tables');
    // get instance of the table
    return JTable::getInstance($type, $prefix, $config);
  }
	/**
	 * Method to auto-populate the model state.
	 *
	 * @return  void
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		$app = JFactory::getApplication();

		// Adjust the context to support modal layouts.
		if ($layout = $app->input->get('layout'))
		{
			$this->context .= '.' . $layout;
		}


		// stupid simple after I figured it out
		$category = $this->getUserStateFromRequest($this->context . '.filter.category', 'filter_category');
		$this->setState('filter.category', $category);

		$vote_year = $this->getUserStateFromRequest($this->context . '.filter.vote_year', 'vote_year');
		$this->setState('filter.vote_year', $vote_year);

		// List state information.
		parent::populateState($ordering, $direction);
	}

	/**
	 * Method to build an SQL query to load the list data.
	 *
	 * @return      string  An SQL query
	 */
	protected function getListQuery()
	{
		// Get the current user for authorisation checks
		$this->user = JFactory::getUser();
		$this->userId = $this->user->get('id');
		$this->guest = $this->user->get('guest');
		$this->groups = $this->user->get('groups');
		$this->authorisedGroups = $this->user->getAuthorisedGroups();
		$this->levels = $this->user->getAuthorisedViewLevels();
		$this->app = JFactory::getApplication();
		$this->input = $this->app->input;
		$this->initSet = true;
		$this->type = $this->input->get('type', ' ', 'STRING');
		// Get a db connection.
		$db = JFactory::getDbo();

		// Create a new query object.
		$query = $db->getQuery(true);

		// Get from #__award_manager_nominee as a
		$query->select($db->quoteName(
			array('a.id','a.firstname','a.lastname','a.imdbinfo','a.mpthree','a.lifestyle_image','a.headshot_image','a.projlink','a.info','a.roletype','a.projage','a.title','a.type','a.character','a.ytvimlink','a.published','a.created','a.modified','a.ordering', 'a.catid'),
			array('id','firstname','lastname','imdbinfo','mpthree','lifestyle_image','headshot_image','projlink','info','roletype','projage','title','type','character','ytvimlink','published','created','modified','ordering', 'catid')));
		$query->from($db->quoteName('#__award_manager_nominee', 'a'));

		// Join over the categories.
		$query->select($db->quoteName(
			array('c.title','c.path'),
			array('category_title','category_path')))
			->join('LEFT', $db->quoteName('#__categories', 'c') . ' ON c.id = a.catid');

		// Get where a.published is 1
		$query->where('a.published = 1');
/*
		$filter_type = $this->getState("filter.type");
	if($filter_type != '0') {
			$query->where("a.type = ".$db->Quote((string) $filter_type));
		} elseif ($this->type != '0') {
			$query->where('a.type = "'.$this->type.'"');
		}*/

		$vote_year = $this->getState('filter.vote_year');
		$query->where('YEAR(a.publish_up) = '.$db->quote($vote_year));
		// Filter by Category.
		if ($category = $this->getState('filter.category'))
		{
			$query->where('a.catid = ' . $db->quote($db->escape($category)));
		}

		$query->order('a.lastname ASC');

		// return the query object
		return $query;
	}

	/**
	 * Method to get an array of data items.
	 *
	 * @return  mixed  An array of data items on success, false on failure.
	 */
	public function getItems()
	{
		$user = JFactory::getUser();
		// check if this user has permission to access item
		if (!$user->authorise('site.nominees.access', 'com_award_manager'))
		{
			$app = JFactory::getApplication();
			$app->enqueueMessage(JText::_('COM_AWARD_MANAGER_NOT_AUTHORISED_TO_VIEW_NOMINEES'), 'error');
			// redirect away to the home page if no access allowed.
			$app->redirect(JURI::root());
			return false;
		}
		// load parent items
		$items = parent::getItems();

		$this->parseVotes($items);

		// Get the global params
		$globalParams = JComponentHelper::getParams('com_award_manager', true);

		// Get the basic encryption.
		$basickey = Award_managerHelper::getCryptKey('basic');
		// Get the encryption object.
		$basic = new FOFEncryptAes($basickey);

		// Insure all item fields are adapted where needed.
		if (Award_managerHelper::checkArray($items))
		{
			foreach ($items as $nr => &$item)
			{
				// Always create a slug for sef URL's
				$item->slug = (isset($item->alias) && isset($item->id)) ? $item->id.':'.$item->alias : $item->id;
				// Check if we can decode lifestyle_image
				if (!empty($item->lifestyle_image) && $basickey && !is_numeric($item->lifestyle_image) && $item->lifestyle_image === base64_encode(base64_decode($item->lifestyle_image, true)))
				{
					// Decode lifestyle_image
					$item->lifestyle_image = rtrim($basic->decryptString($item->lifestyle_image), "\0");
				}
				// Check if we can decode headshot_image
				if (!empty($item->headshot_image) && $basickey && !is_numeric($item->headshot_image) && $item->headshot_image === base64_encode(base64_decode($item->headshot_image, true)))
				{
					// Decode headshot_image
					$item->headshot_image = rtrim($basic->decryptString($item->headshot_image), "\0");
				}
			}
		}

		// return items
		return $items;
	}

		private function parseVotes($items) {

			$model = Award_managerHelper::getModel('votes', JPATH_ROOT.  '/components/com_award_manager', 'Award_manager');
			$user = JFactory::getUser();
			$ids = array();
			$cats = array();

			foreach ($items as $item) {
					$ids[] = $item->id;
					$cats[] = $item->catid;
			}

			//$totals = $repo->countForResources('post', $ids);
			$votes = $model->findByUserForResources($user->id, $cats, $ids);

			foreach ($items as $item) {
					//$post->likes = new stdClass();

					//$post->likes->total = isset($totals[$post->id]) ? $totals[$post->id]['likes'] : 0;
					$item->voted = isset($votes[$item->id]);
			}
	}

	public function findWinners($catid,$wyear/*array $cats = array()*/)
	{
			$dbo = $this->getDbo();
			$winner = $this->getTable();
			//$profile = $this->getTable('member', 'Award_managerTable', array());

			$query = $dbo->getQuery(true)
					->select($dbo->quoteName(
						array('w.catid','w.firstname','w.lastname'),array('id','firstname','lastname')))
					->from($dbo->qn($winner->getTableName(), 'w'));

			$query->where($dbo->qn('w.catid') . ' = ' . $dbo->q($catid));
			//$query->where($dbo->qn('w.catid') . ' IN (' . implode(',', $dbo->q($cats)) . ')');
					$query->where('w.winner = 1');
					$query->where('YEAR(w.publish_up) = '.$dbo->q($wyear));
					$query->where('w.published = 1');



			$result = $dbo->setQuery($query)
					->loadObjectList();

			return $result;
	}

	public function findNominees($catid, $nyear/*array $cats = array()*/)
	{
			$dbo = $this->getDbo();
			$nominee = $this->getTable();
			//$profile = $this->getTable('member', 'Award_managerTable', array());

			$query = $dbo->getQuery(true)
					->select($dbo->quoteName(
						array('nom.catid','nom.firstname','nom.lastname', 'nom.title'),array('id','firstname','lastname', 'title')))
					->from($dbo->qn($nominee->getTableName(), 'nom'));

			$query->where($dbo->qn('nom.catid') . ' = ' . $dbo->q($catid));
			//$query->where($dbo->qn('w.catid') . ' IN (' . implode(',', $dbo->q($cats)) . ')');
					$query->where('YEAR(nom.publish_up) = '.$dbo->q($nyear));
					$query->where('nom.published = 1');

			$result = $dbo->setQuery($query)
					->loadObjectList();

			return $result;
	}

	public function countNominees($cats,$nyear)
	{
		$nominee = $this->getTable();
		$dbo = $this->getDbo();

			//$categories = $this->getTable('categories', 'Award_managerTable', array());
			if (!$cats) {
					return array();
			}

			$query = $dbo->getQuery(true)
					->select($dbo->qn('n.catid'))
					->select('COUNT(1) AS nominees')
					->from($dbo->qn($nominee->getTableName(), 'n'))
					->where('YEAR(n.publish_up) = '.$dbo->q($nyear))
					->where($dbo->qn('n.catid') . ' IN (' . implode(',', $dbo->q($cats)) . ')')
					//->where($dbo->qn('c.id') . ' IN (' . implode(',', $dbo->q($ccats)) . ')')
					->group($dbo->qn('n.catid'));

			$results = $dbo->setQuery($query)->loadAssocList('catid');

			return $results;
	}

	public function countCategories(array $cats = array())
	{
			$winners = $this->getTable();
			//$categories = $this->getTable('categories', 'Award_managerTable', array());

			if (!$cats) {
					return array();
			}

			$dbo = $this->getDbo();
			$query = $dbo->getQuery(true)
					->select($dbo->qn('c.id'))
					->select('COUNT(1) AS categories')
					->from($dbo->qn('#__categories', 'c'))
					->where($dbo->qn('c.parent_id') . ' IN (' . implode(',', $dbo->q($cats)) . ')')
					//->where($dbo->qn('c.id') . ' IN (' . implode(',', $dbo->q($ccats)) . ')')
					->group($dbo->qn('c.id'));

			$results = $dbo->setQuery($query)
					->loadAssocList('id');

			return $results;
	}

	public function getCategoryName()
	{
		$catid = $this->getState('filter.category');
		$categories = JCategories::getInstance('Award_manager', array());
		$categoryNode = $categories->get($catid);
		return $categoryNode->path;
	}

}
